@@ -132,6 +132,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName AP = new COSName( "AP" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName APP = new COSName( "App" );
 
     /**
      * A common COSName value.
@@ -300,6 +304,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName COLUMNS = new COSName( "Columns" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName CONTACT_INFO = new COSName( "ContactInfo" );
     /**
      * A common COSName value.
      */
@@ -340,6 +348,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName CS = new COSName(  "CS" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName DEFAULT = new COSName( "default" );
     /**
      * A common COSName value.
      */
@@ -348,6 +360,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName DA = new COSName( "DA" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName DATE = new COSName( "Date" );
     /**
      * A common COSName value.
      */
@@ -404,6 +420,34 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName DIFFERENCES = new COSName( "Differences" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName DIGEST_METHOD = new COSName( "DigestMethod" );
+    /**
+     * Digest Method.
+     */
+    public static final COSName DIGEST_SHA1 = new COSName("SHA1");
+
+    /**
+     * Digest Method.
+     */
+    public static final COSName DIGEST_SHA256 = new COSName("SHA256");
+
+    /**
+     * Digest Method.
+     */
+    public static final COSName DIGEST_SHA384 = new COSName("SHA384");
+
+    /**
+     * Digest Method.
+     */
+    public static final COSName DIGEST_SHA512 = new COSName("SHA512");
+
+    /**
+     * Digest Method.
+     */
+    public static final COSName DIGEST_RIPEMD160 = new COSName("RIPEMD160");
     /**
      * A common COSName value.
      */
@@ -422,11 +466,19 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName DOC_CHECKSUM = new COSName( "DocChecksum" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName DOC_TIME_STAMP = new COSName( "DocTimeStamp" );
 
     /**
      * A common COSName value.
      */
     public static final COSName DOMAIN = new COSName( "Domain" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName DOS = new COSName( "DOS" );
     /**
      * A common COSName value.
      */
@@ -453,6 +505,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName E = new COSName("E");
+    /**
+     * A common COSName value.
+     */
+    public static final COSName EF = new COSName("EF");
 
     /**
      * A common COSName value.
@@ -540,10 +596,18 @@ public final class COSName extends COSBase implements Comparable<COSName>
     /**
     * A common COSName value.
     */
+    public static final COSName FILESPEC = new COSName( "Filespec" );
+    /**
+    * A common COSName value.
+    */
     public static final COSName FILTER = new COSName( "Filter" );
     /**
     * A common COSName value.
     */
+    public static final COSName FIRST = new COSName( "First" );
+    /**
+    * A common COSName value.
+    */
     public static final COSName FIRST_CHAR = new COSName( "FirstChar" );
     /**
      * A common COSName value.
@@ -666,6 +730,11 @@ public final class COSName extends COSBase implements Comparable<COSName>
     */
     public static final COSName ICCBASED = new COSName( "ICCBased" );
 
+    /**
+     * A common COSName value.
+     */
+    public static final COSName I = new COSName("I");
+
     /**
      * A common COSName value.
      */
@@ -720,6 +789,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName JBIG2_DECODE = new COSName( "JBIG2Decode" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName JBIG2_GLOBALS = new COSName( "JBIG2Globals" );
     /**
     * A common COSName value.
     */
@@ -762,6 +835,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName LC = new COSName( "LC" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName L = new COSName( "L" );
     /**
      * A common COSName value.
      */
@@ -769,12 +846,17 @@ public final class COSName extends COSBase implements Comparable<COSName>
     /**
     * A common COSName value.
     */
+    public static final COSName LEGAL_ATTESTATION = new COSName( "LegalAttestation" );
+    /**
+    * A common COSName value.
+    */
     public static final COSName LENGTH = new COSName( "Length" );
     /**
      * A common COSName value.
      */
     public static final COSName LENGTH1 = new COSName( "Length1" );
 
+    public static final COSName LENGTH2 = new COSName( "Length2" );
     /**
      * A common COSName value.
      */
@@ -802,6 +884,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
     /**
     * A common COSName value.
     */
+    public static final COSName MAC = new COSName( "Mac" );
+    /**
+    * A common COSName value.
+    */
     public static final COSName MAC_ROMAN_ENCODING = new COSName( "MacRomanEncoding" );
 
     /**
@@ -825,6 +911,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName MCID = new COSName("MCID");
+    /**
+     * A common COSName value.
+     */
+    public static final COSName MDP = new COSName("MDP");
     /**
      * A common COSName value.
      */
@@ -871,6 +961,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName NM = new COSName( "NM" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName NON_EFONT_NO_WARN = new COSName( "NonEFontNoWarn" );
     /**
      * A common COSName value.
      */
@@ -924,6 +1018,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName OPT = new COSName( "Opt" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName OS = new COSName( "OS" );
     /**
      * A common COSName value.
      */
@@ -997,6 +1095,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName PG = new COSName("Pg");
+    /**
+     * A common COSName value.
+     */
+    public static final COSName PRE_RELEASE = new COSName( "PreRelease" );
     /**
      * A common COSName value.
      */
@@ -1027,9 +1129,17 @@ public final class COSName extends COSBase implements Comparable<COSName>
      */
     public static final COSName PRODUCER = new COSName( "Producer" );
 
+    /**
+     * A common COSName value.
+     */
+    public static final COSName PROP_BUILD = new COSName( "Prop_Build" );
     /** The COSName value for "Properties". */
     public static final COSName PROPERTIES = new COSName( "Properties" );
 
+    /**
+     * A common COSName value.
+     */
+    public static final COSName PUB_SEC = new COSName( "PubSec" );
     /**
      * A common COSName value.
      */
@@ -1042,6 +1152,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName RANGE = new COSName( "Range" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName REASONS = new COSName( "Reasons" );
     /**
      * A common COSName value.
      */
@@ -1190,6 +1304,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName SUBTYPE = new COSName( "Subtype" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName SV = new COSName( "SV" );
 
     /**
      * A common COSName value.
@@ -1210,6 +1328,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName TILING_TYPE = new COSName( "TilingType" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName TIME_STAMP = new COSName( "TimeStamp" );
     /**
      * A common COSName value.
      */
@@ -1230,6 +1352,10 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName TRUE_TYPE = new COSName("TrueType" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName TRUSTED_MODE = new COSName("TrustedMode" );
     /**
     * A common COSName value.
     */
@@ -1259,12 +1385,24 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName U = new COSName( "U" );
+    /**
+     * A common COSName value.
+     */
+    public static final COSName UF = new COSName( "UF" );
     /** the COSName for the "Unchanged" value. */
     public static final COSName UNCHANGED = new COSName("Unchanged");
+    /**
+     * A common COSName value.
+     */
+    public static final COSName UNIX = new COSName( "Unix" );
     /**
      * A common COSName value.
      */
     public static final COSName URI = new COSName("URI");
+    /**
+     * A common COSName value.
+     */
+    public static final COSName URL = new COSName("URL");
 
     /**
     * A common COSName value.
@@ -1326,6 +1464,11 @@ public final class COSName extends COSBase implements Comparable<COSName>
      * A common COSName value.
      */
     public static final COSName XREF = new COSName( "XRef" );
+
+    /**
+     * A common COSName value.
+     */
+    public static final COSName XREF_STM = new COSName( "XRefStm" );
     /**
      * A common COSName value.
      */
