@@ -27,7 +27,6 @@ import java.text.NumberFormat;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Locale;
-import java.util.Map;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -51,11 +50,10 @@ import org.apache.pdfbox.pdmodel.graphics.color.PDPattern;
 import org.apache.pdfbox.pdmodel.graphics.color.PDSeparation;
 import org.apache.pdfbox.pdmodel.graphics.xobject.PDXObject;
 import org.apache.pdfbox.pdmodel.graphics.xobject.PDXObjectImage;
-import org.apache.pdfbox.util.MapUtil;
 
 
 /**
- * This class will is a convenience for creating page content streams.  You MUST
+ * This class is a convenience for creating page content streams.  You MUST
  * call close() when you are finished with this object.
  *
  * @author <a href="mailto:ben@benlitchfield.com">Ben Litchfield</a>
@@ -66,16 +64,12 @@ public class PDPageContentStream
     /**
      * Log instance.
      */
-    private static final Log log = LogFactory.getLog(PDPageContentStream.class);
+    private static final Log LOG = LogFactory.getLog(PDPageContentStream.class);
 
     private PDPage page;
     private OutputStream output;
     private boolean inTextMode = false;
-    private Map<PDFont,String> fontMappings;
-    private Map<PDXObject,String> xobjectMappings;
     private PDResources resources;
-    private Map fonts;
-    private Map xobjects;
 
     private PDColorSpace currentStrokingColorSpace = new PDDeviceGray();
     private PDColorSpace currentNonStrokingColorSpace = new PDDeviceGray();
@@ -110,6 +104,9 @@ public class PDPageContentStream
     private static final String CLOSE_STROKE = "s\n";
     private static final String STROKE = "S\n";
     private static final String LINE_WIDTH = "w\n";
+    private static final String LINE_JOIN_STYLE = "j\n";
+    private static final String LINE_CAP_STYLE = "J\n";
+    private static final String LINE_DASH_PATTERN = "d\n";
     private static final String CLOSE_SUBPATH = "h\n";
     private static final String CLIP_PATH_NON_ZERO = "W\n";
     private static final String CLIP_PATH_EVEN_ODD = "W*\n";
@@ -118,8 +115,6 @@ public class PDPageContentStream
     private static final String BEZIER_32 = "v\n";
     private static final String BEZIER_313 = "y\n";
 
-    private static final String MP = "MP\n";
-    private static final String DP = "DP\n";
     private static final String BMC = "BMC\n";
     private static final String BDC = "BDC\n";
     private static final String EMC = "EMC\n";
@@ -173,7 +168,8 @@ public class PDPageContentStream
      * @param resetContext Tell if the graphic context should be reseted.
      * @throws IOException If there is an error writing to the page contents.
      */
-    public PDPageContentStream( PDDocument document, PDPage sourcePage, boolean appendContent, boolean compress, boolean resetContext )
+    public PDPageContentStream( PDDocument document, PDPage sourcePage, boolean appendContent, 
+            boolean compress, boolean resetContext )
             throws IOException
     {
         
@@ -185,14 +181,6 @@ public class PDPageContentStream
             page.setResources( resources );
         }
 
-        //Fonts including reverse lookup
-        fonts = resources.getFonts();
-        fontMappings = reverseMap(fonts, PDFont.class);
-
-        //XObjects including reverse lookup
-        xobjects = resources.getXObjects();
-        xobjectMappings = reverseMap(xobjects, PDXObject.class);
-
         // Get the pdstream from the source page instead of creating a new one
         PDStream contents = sourcePage.getContents();
         boolean hasContent = contents != null;
@@ -260,7 +248,7 @@ public class PDPageContentStream
         {
             if (hasContent)
             {
-                log.warn("You are overwriting an existing content, you should use the append mode");
+                LOG.warn("You are overwriting an existing content, you should use the append mode");
             }
             contents = new PDStream( document );
             if( compress )
@@ -276,17 +264,6 @@ public class PDPageContentStream
         formatDecimal.setGroupingUsed( false );
     }
 
-    private <T> Map<T, String> reverseMap(Map map, Class<T> keyClass)
-    {
-        Map<T, String> reversed = new java.util.HashMap<T, String>();
-        for (Object o : map.entrySet())
-        {
-            Map.Entry entry = (Map.Entry)o;
-            reversed.put(keyClass.cast(entry.getValue()), (String)entry.getKey());
-        }
-        return reversed;
-    }
-
     /**
      * Begin some text operations.
      *
@@ -328,13 +305,7 @@ public class PDPageContentStream
      */
     public void setFont( PDFont font, float fontSize ) throws IOException
     {
-        String fontMapping = fontMappings.get( font );
-        if( fontMapping == null )
-        {
-            fontMapping = MapUtil.getNextUniqueKey( fonts, "F" );
-            fontMappings.put( font, fontMapping );
-            fonts.put( fontMapping, font );
-        }
+        String fontMapping = resources.addFont(font);
         appendRawCommands( "/");
         appendRawCommands( fontMapping );
         appendRawCommands( SPACE );
@@ -384,6 +355,10 @@ public class PDPageContentStream
      */
     public void drawXObject( PDXObject xobject, AffineTransform transform ) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: drawXObject is not allowed within a text block." );
+        }
         String xObjectPrefix = null;
         if( xobject instanceof PDXObjectImage )
         {
@@ -393,14 +368,7 @@ public class PDPageContentStream
         {
             xObjectPrefix = "Form";
         }
-
-        String objMapping = xobjectMappings.get( xobject );
-        if( objMapping == null )
-        {
-            objMapping = MapUtil.getNextUniqueKey( xobjects, xObjectPrefix );
-            xobjectMappings.put( xobject, objMapping );
-            xobjects.put( objMapping, xobject );
-        }
+        String objMapping = resources.addXObject(xobject, xObjectPrefix);
         saveGraphicsState();
         appendRawCommands( SPACE );
         concatenate2CTM(transform);
@@ -473,6 +441,10 @@ public class PDPageContentStream
     */
     public void setTextMatrix(AffineTransform matrix) throws IOException
     {
+        if( !inTextMode )
+        {
+            throw new IOException( "Error: must call beginText() before setTextMatrix");
+        }
         appendMatrix(matrix);
         appendRawCommands(SET_TEXT_MATRIX);
     }
@@ -688,7 +660,7 @@ public class PDPageContentStream
         else if( colorSpace.getType() == ColorSpace.TYPE_CMYK )
         {
             color.getColorComponents( colorComponents );
-            setStrokingColor( colorComponents[0], colorComponents[2], colorComponents[2], colorComponents[3] );
+            setStrokingColor( colorComponents[0], colorComponents[1], colorComponents[2], colorComponents[3] );
         }
         else
         {
@@ -717,7 +689,7 @@ public class PDPageContentStream
         else if( colorSpace.getType() == ColorSpace.TYPE_CMYK )
         {
             color.getColorComponents( colorComponents );
-            setNonStrokingColor( colorComponents[0], colorComponents[2], colorComponents[2], colorComponents[3] );
+            setNonStrokingColor( colorComponents[0], colorComponents[1], colorComponents[2], colorComponents[3] );
         }
         else
         {
@@ -940,6 +912,10 @@ public class PDPageContentStream
      */
     public void addRect( float x, float y, float width, float height ) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: addRect is not allowed within a text block." );
+        }
         appendRawCommands( formatDecimal.format( x ) );
         appendRawCommands( SPACE );
         appendRawCommands( formatDecimal.format( y ) );
@@ -962,6 +938,10 @@ public class PDPageContentStream
      */
     public void fillRect( float x, float y, float width, float height ) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: fillRect is not allowed within a text block." );
+        }
         addRect(x, y, width, height);
         fill(PathIterator.WIND_NON_ZERO);
     }
@@ -979,6 +959,10 @@ public class PDPageContentStream
      */
     public void addBezier312(float x1, float y1, float x2, float y2, float x3, float y3) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: addBezier312 is not allowed within a text block." );
+        }
         appendRawCommands( formatDecimal.format( x1) );
         appendRawCommands( SPACE );
         appendRawCommands( formatDecimal.format( y1) );
@@ -1005,6 +989,10 @@ public class PDPageContentStream
      */
     public void addBezier32(float x2, float y2, float x3, float y3) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: addBezier32 is not allowed within a text block." );
+        }
         appendRawCommands( formatDecimal.format( x2) );
         appendRawCommands( SPACE );
         appendRawCommands( formatDecimal.format( y2) );
@@ -1027,6 +1015,10 @@ public class PDPageContentStream
      */
     public void addBezier31(float x1, float y1, float x3, float y3) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: addBezier31 is not allowed within a text block." );
+        }
         appendRawCommands( formatDecimal.format( x1) );
         appendRawCommands( SPACE );
         appendRawCommands( formatDecimal.format( y1) );
@@ -1048,7 +1040,10 @@ public class PDPageContentStream
      */
     public void moveTo( float x, float y) throws IOException
     {
-        // moveTo
+        if( inTextMode )
+        {
+            throw new IOException( "Error: moveTo is not allowed within a text block." );
+        }
         appendRawCommands( formatDecimal.format( x) );
         appendRawCommands( SPACE );
         appendRawCommands( formatDecimal.format( y) );
@@ -1065,7 +1060,10 @@ public class PDPageContentStream
      */
     public void lineTo( float x, float y) throws IOException
     {
-        // moveTo
+        if( inTextMode )
+        {
+            throw new IOException( "Error: lineTo is not allowed within a text block." );
+        }
         appendRawCommands( formatDecimal.format( x) );
         appendRawCommands( SPACE );
         appendRawCommands( formatDecimal.format( y) );
@@ -1083,6 +1081,10 @@ public class PDPageContentStream
      */
     public void addLine( float xStart, float yStart, float xEnd, float yEnd ) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: addLine is not allowed within a text block." );
+        }
         // moveTo
         moveTo(xStart, yStart);
         // lineTo
@@ -1100,6 +1102,10 @@ public class PDPageContentStream
      */
     public void drawLine( float xStart, float yStart, float xEnd, float yEnd ) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: drawLine is not allowed within a text block." );
+        }
         addLine(xStart, yStart, xEnd, yEnd);
         // stroke
         stroke();
@@ -1113,6 +1119,10 @@ public class PDPageContentStream
      */
     public void addPolygon(float[] x, float[] y) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: addPolygon is not allowed within a text block." );
+        }
         if (x.length != y.length)
         {
             throw new IOException( "Error: some points are missing coordinate" );
@@ -1139,6 +1149,10 @@ public class PDPageContentStream
      */
     public void drawPolygon(float[] x, float[] y) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: drawPolygon is not allowed within a text block." );
+        }
         addPolygon(x, y);
         stroke();
     }
@@ -1151,31 +1165,55 @@ public class PDPageContentStream
      */
     public void fillPolygon(float[] x, float[] y) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: fillPolygon is not allowed within a text block." );
+        }
         addPolygon(x, y);
         fill(PathIterator.WIND_NON_ZERO);
     }
 
     /**
      * Stroke the path.
+     * 
+     * @throws IOException If there is an error while stroking the path.
      */
     public void stroke() throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: stroke is not allowed within a text block." );
+        }
         appendRawCommands( STROKE );
     }
 
     /**
      * Close and stroke the path.
+     * 
+     * @throws IOException If there is an error while closing and stroking the path.
      */
     public void closeAndStroke() throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: closeAndStroke is not allowed within a text block." );
+        }
         appendRawCommands( CLOSE_STROKE );
     }
 
     /**
      * Fill the path.
+     * 
+     * @param windingRule the winding rule to be used for filling 
+     * 
+     * @throws IOException If there is an error while filling the path.
      */
     public void fill(int windingRule) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: fill is not allowed within a text block." );
+        }
         if (windingRule == PathIterator.WIND_NON_ZERO)
         {
             appendRawCommands( FILL_NON_ZERO );
@@ -1193,17 +1231,31 @@ public class PDPageContentStream
 
     /**
      * Close subpath.
+     * 
+     * @throws IOException If there is an error while closing the subpath.
      */
     public void closeSubPath() throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: closeSubPath is not allowed within a text block." );
+        }
         appendRawCommands( CLOSE_SUBPATH );
     }
 
     /**
      * Clip path.
+     * 
+     * @param windingRule the winding rule to be used for clipping
+     *  
+     * @throws IOException If there is an error while clipping the path.
      */
     public void clipPath(int windingRule) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: clipPath is not allowed within a text block." );
+        }
         if (windingRule == PathIterator.WIND_NON_ZERO)
         {
             appendRawCommands( CLIP_PATH_NON_ZERO );
@@ -1228,10 +1280,85 @@ public class PDPageContentStream
      */
     public void setLineWidth(float lineWidth) throws IOException
     {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: setLineWidth is not allowed within a text block." );
+        }
         appendRawCommands( formatDecimal.format( lineWidth ) );
         appendRawCommands( SPACE );
         appendRawCommands( LINE_WIDTH );
     }
+    
+    /**
+     * Set the line join style.
+     * @param lineJoinStyle 0 for miter join, 1 for round join, and 2 for bevel join.
+     * @throws IOException If there is an error while writing to the stream.
+     */
+    public void setLineJoinStyle(int lineJoinStyle) throws IOException
+    {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: setLineJoinStyle is not allowed within a text block." );
+        }
+        if (lineJoinStyle >= 0 && lineJoinStyle <= 2)
+        {
+            appendRawCommands( Integer.toString( lineJoinStyle ) );
+            appendRawCommands( SPACE );
+            appendRawCommands( LINE_JOIN_STYLE );
+        }
+        else
+        {
+            throw new IOException( "Error: unknown value for line join style" );
+        }
+    }
+    
+    
+    /**
+     * Set the line cap style.
+     * @param lineCapStyle 0 for butt cap, 1 for round cap, and 2 for projecting square cap.
+     * @throws IOException If there is an error while writing to the stream.
+     */
+    public void setLineCapStyle(int lineCapStyle) throws IOException
+    {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: setLineCapStyle is not allowed within a text block." );
+        }
+        if (lineCapStyle >= 0 && lineCapStyle <= 2)
+        {
+            appendRawCommands( Integer.toString( lineCapStyle ) );
+            appendRawCommands( SPACE );
+            appendRawCommands( LINE_CAP_STYLE );
+        }
+        else
+        {
+            throw new IOException( "Error: unknown value for line cap style" );
+        }
+    }
+    
+    /**
+     * Set the line dash pattern.
+     * @param pattern The pattern array
+     * @param phase The phase of the pattern
+     * @throws IOException If there is an error while writing to the stream.
+     */
+    public void setLineDashPattern(float[] pattern, float phase) throws IOException
+    {
+        if( inTextMode )
+        {
+            throw new IOException( "Error: setLineDashPattern is not allowed within a text block." );
+        }
+        appendRawCommands( "[" );
+        for (float value : pattern)
+        {
+            appendRawCommands( formatDecimal.format( value ) );
+            appendRawCommands( SPACE );
+        }
+        appendRawCommands( "] ");
+        appendRawCommands( formatDecimal.format(phase) );
+        appendRawCommands( SPACE );
+        appendRawCommands( LINE_DASH_PATTERN );
+    }
 
     /**
      * Begin a marked content sequence.
