@@ -70,28 +70,33 @@ public abstract class PDFont implements COSObjectable
     private Encoding fontEncoding = null;
 
     /**
-     *  The descriptor of the font
+     *  The descriptor of the font.
      */
     private PDFontDescriptor fontDescriptor = null;
 
     /**
-     *  The font matrix
+     *  The font matrix.
      */
     protected PDMatrix fontMatrix = null;
-    
+
     /**
      * This is only used if this is a font object and it has an encoding and it is
      * a type0 font with a cmap.
      */
     protected CMap cmap = null;
+
+    /**
+     * The CMap holding the ToUnicode mapping.
+     */
+    protected CMap toUnicodeCmap = null;
     
     private boolean hasToUnicode = false;
 
     protected static Map<String, CMap> cmapObjects =
         Collections.synchronizedMap( new HashMap<String, CMap>() );
-    
+
     /**
-     *  A list a floats representing the widths
+     *  A list a floats representing the widths.
      */
     private List<Float> widths = null;
 
@@ -103,7 +108,8 @@ public abstract class PDFont implements COSObjectable
 
     // TODO move the Map to PDType1Font as these are the 14 Standard fonts
     // which are definitely Type 1 fonts
-    private static Map<String, FontMetric> getAdobeFontMetrics() {
+    private static Map<String, FontMetric> getAdobeFontMetrics()
+    {
         Map<String, FontMetric> metrics = new HashMap<String, FontMetric>();
         addAdobeFontMetric( metrics, "Courier-Bold" );
         addAdobeFontMetric( metrics, "Courier-BoldOblique" );
@@ -122,25 +128,32 @@ public abstract class PDFont implements COSObjectable
         return metrics;
     }
 
-    protected final static String resourceRootCMAP = "org/apache/pdfbox/resources/cmap/";
-    private final static String resourceRootAFM = "org/apache/pdfbox/resources/afm/";
+    protected static final String resourceRootCMAP = "org/apache/pdfbox/resources/cmap/";
+    private static final String resourceRootAFM = "org/apache/pdfbox/resources/afm/";
 
     private static void addAdobeFontMetric(
-            Map<String, FontMetric> metrics, String name ) {
-        try {
+            Map<String, FontMetric> metrics, String name )
+    {
+        try
+        {
             String resource = resourceRootAFM + name + ".afm";
             InputStream afmStream = ResourceLoader.loadResource( resource );
             if( afmStream != null )
             {
-                try {
+                try
+                {
                     AFMParser parser = new AFMParser( afmStream );
                     parser.parse();
                     metrics.put( name, parser.getResult() );
-                } finally {
+                }
+                finally
+                {
                     afmStream.close();
                 }
             }
-        } catch (Exception e) {
+        }
+        catch (Exception e)
+        {
             // ignore
         }
     }
@@ -192,13 +205,13 @@ public abstract class PDFont implements COSObjectable
         if(fontDescriptor == null)
         {
             COSDictionary fd = (COSDictionary)font.getDictionaryObject( COSName.FONT_DESC );
-            if (fd != null) 
+            if (fd != null)
             {
                 fontDescriptor = new PDFontDescriptorDictionary( fd );
             }
             else
             {
-                FontMetric afm = getAFM();
+                getAFM();
                 if( afm != null )
                 {
                     fontDescriptor = new PDFontDescriptorAFM( afm );
@@ -207,30 +220,30 @@ public abstract class PDFont implements COSObjectable
         }
         return fontDescriptor;
     }
-    
+
     /**
      * This will set the font descriptor.
      *
-     * @param fontDescriptor The font descriptor.
+     * @param fdDictionary The font descriptor.
      */
-    public void setFontDescriptor( PDFontDescriptorDictionary fontDescriptor )
+    public void setFontDescriptor( PDFontDescriptorDictionary fdDictionary )
     {
         COSDictionary dic = null;
-        if( fontDescriptor != null )
+        if( fdDictionary != null )
         {
-            dic = fontDescriptor.getCOSDictionary();
+            dic = fdDictionary.getCOSDictionary();
         }
         font.setItem( COSName.FONT_DESC, dic );
-        this.fontDescriptor = fontDescriptor;
+        fontDescriptor = fdDictionary;
     }
-    
+
     /**
      * Determines the encoding for the font.
-     * This method as to be overwritten, as there are different 
+     * This method as to be overwritten, as there are different
      * possibilities to define a mapping.
      */
-   	protected abstract void determineEncoding();
-    
+    protected abstract void determineEncoding();
+
     /**
      * {@inheritDoc}
      */
@@ -300,15 +313,35 @@ public abstract class PDFont implements COSObjectable
      * @param string The string to draw.
      * @param g The graphics to draw onto.
      * @param fontSize The size of the font to draw.
-     * @param at The transformation matrix with all infos for scaling and shearing of the font.
+     * @param at The transformation matrix with all information for scaling and shearing of the font.
+     * @param x The x coordinate to draw at.
+     * @param y The y coordinate to draw at.
+     *
+     * @throws IOException If there is an error drawing the specific string.
+     * @deprecated use {@link PDFont#drawString(String, int[], Graphics, float, AffineTransform, float, float)} instead
+     */
+    public void drawString( String string, Graphics g, float fontSize, AffineTransform at, float x, float y ) 
+    throws IOException
+    {
+        drawString(string, null, g, fontSize, at, x, y);
+    }
+
+    /**
+     * This will draw a string on a canvas using the font.
+     *
+     * @param string The string to draw.
+     * @param codePoints The codePoints of the given string.
+     * @param g The graphics to draw onto.
+     * @param fontSize The size of the font to draw.
+     * @param at The transformation matrix with all information for scaling and shearing of the font.
      * @param x The x coordinate to draw at.
      * @param y The y coordinate to draw at.
      *
      * @throws IOException If there is an error drawing the specific string.
      */
-    public abstract void drawString( String string, Graphics g, float fontSize,
+    public abstract void drawString( String string, int[] codePoints, Graphics g, float fontSize,
         AffineTransform at, float x, float y ) throws IOException;
-
+    
     /**
      * Used for multibyte encodings.
      *
@@ -318,7 +351,7 @@ public abstract class PDFont implements COSObjectable
      *
      * @return The int value of data from the array.
      */
-    protected int getCodeFromArray( byte[] data, int offset, int length )
+    public int getCodeFromArray( byte[] data, int offset, int length )
     {
         int code = 0;
         for( int i=0; i<length; i++ )
@@ -344,8 +377,7 @@ public abstract class PDFont implements COSObjectable
         FontMetric metric = getAFM();
         if( metric != null )
         {
-            Encoding encoding = getFontEncoding();
-            String characterName = encoding.getName( code );
+            String characterName = fontEncoding.getName( code );
             retval = metric.getCharacterWidth( characterName );
         }
         return retval;
@@ -377,7 +409,8 @@ public abstract class PDFont implements COSObjectable
      */
     protected FontMetric getAFM()
     {
-        if(isType1Font() && afm==null){
+        if (isType1Font() && afm == null)
+        {
             COSBase baseFont = font.getDictionaryObject( COSName.BASE_FONT );
             String name = null;
             if( baseFont instanceof COSName )
@@ -389,12 +422,12 @@ public abstract class PDFont implements COSObjectable
                 }
 
             }
-            else if( baseFont instanceof COSString )
+            else if ( baseFont instanceof COSString )
             {
                 COSString string = (COSString)baseFont;
                 name = string.getString();
             }
-            if( name != null )
+            if ( name != null )
             {
                 afm = afmObjects.get( name );
             }
@@ -403,7 +436,7 @@ public abstract class PDFont implements COSObjectable
     }
 
     private FontMetric afm = null;
-    
+
     private COSBase encoding = null;
     /**
      * cache the {@link COSName#ENCODING} object from
@@ -413,43 +446,51 @@ public abstract class PDFont implements COSObjectable
      * <pre>
      *   font.getDictionaryObject(COSName.ENCODING);
      * </pre>
-     * @return
+     * @return the encoding
      */
-    protected COSBase getEncoding(){
-    	if(encoding==null)
-    	{
-    		encoding = font.getDictionaryObject( COSName.ENCODING );
-    	}
-    	return encoding;
+    protected COSBase getEncoding()
+    {
+        if (encoding == null)
+        {
+            encoding = font.getDictionaryObject( COSName.ENCODING );
+        }
+        return encoding;
     }
 
     /**
      * Set the encoding object from the fonts dictionary.
-     * @param encoding the given encoding.
+     * @param encodingValue the given encoding.
      */
-    protected void setEncoding(COSBase encoding){
-        font.setItem( COSName.ENCODING, encoding );
-        this.encoding = encoding;
+    protected void setEncoding(COSBase encodingValue)
+    {
+        font.setItem( COSName.ENCODING, encodingValue );
+        encoding = encodingValue;
     }
-    
+
     /**
      * Encode the given value using the CMap of the font.
-     * 
+     *
      * @param code the code to encode.
      * @param length the byte length of the given code.
      * @param isCIDFont indicates that the used font is a CID font.
-     * 
+     *
      * @return The value of the encoded character.
+     * @throws IOException if something went wrong
      */
-    protected String cmapEncoding( int code, int length, boolean isCIDFont ) throws IOException
+    protected String cmapEncoding( int code, int length, boolean isCIDFont, CMap sourceCmap ) throws IOException
     {
         String retval = null;
-        if (cmap != null)
+        // there is not sourceCmap if this is a descendant font
+        if (sourceCmap == null)
         {
-            retval = cmap.lookup(code, length);
-            if (retval == null && isCIDFont) 
+            sourceCmap = cmap;
+        }
+        if (sourceCmap != null)
+        {
+            retval = sourceCmap.lookup(code, length);
+            if (retval == null && isCIDFont)
             {
-                retval = cmap.lookupCID(code);
+                retval = sourceCmap.lookupCID(code);
             }
         }
         return retval;
@@ -469,18 +510,21 @@ public abstract class PDFont implements COSObjectable
     {
         String retval = null;
         int code = getCodeFromArray( c, offset, length );
-        if( cmap != null )
+        if( toUnicodeCmap != null )
         {
-            retval = cmapEncoding(code, length, false);
+            retval = cmapEncoding(code, length, false, toUnicodeCmap);
         }
-        
+        if( retval == null && cmap != null )
+        {
+            retval = cmapEncoding(code, length, false, cmap);
+        }
+
         // there is no cmap but probably an encoding with a suitable mapping
         if( retval == null )
         {
-            Encoding encoding = getFontEncoding();
-            if( encoding != null )
+            if( fontEncoding != null )
             {
-                retval = encoding.getCharacter( code );
+                retval = fontEncoding.getCharacter( code );
             }
             if( retval == null && (cmap == null || length == 2))
             {
@@ -490,6 +534,16 @@ public abstract class PDFont implements COSObjectable
         return retval;
     }
 
+    public int encodeToCID( byte[] c, int offset, int length ) throws IOException
+    {
+        int code = -1;
+        if (encode(c, offset, length) != null) 
+        {
+            code = getCodeFromArray( c, offset, length );
+        }
+        return code;
+    }
+    
     private static final String[] SINGLE_CHAR_STRING = new String[256];
     private static final String[][] DOUBLE_CHAR_STRING = new String[256][256];
     static
@@ -538,22 +592,24 @@ public abstract class PDFont implements COSObjectable
         return retval;
     }
 
-    protected void parseCmap( String cmapRoot, InputStream cmapStream)
+    protected CMap parseCmap( String cmapRoot, InputStream cmapStream)
     {
+        CMap targetCmap = null;
         if( cmapStream != null )
         {
             CMapParser parser = new CMapParser();
-            try 
+            try
             {
-                cmap = parser.parse( cmapRoot, cmapStream );
+                targetCmap = parser.parse( cmapRoot, cmapStream );
                 // limit the cache to external CMaps
                 if (cmapRoot != null)
                 {
-                    cmapObjects.put( cmap.getName(), cmap );
+                    cmapObjects.put( targetCmap.getName(), targetCmap );
                 }
             }
             catch (IOException exception) {}
         }
+        return targetCmap;
     }
 
     /**
@@ -591,6 +647,7 @@ public abstract class PDFont implements COSObjectable
     private boolean type1Font;
     private boolean trueTypeFont;
     private boolean typeFont;
+    private boolean type0Font;
 
     /**
      * This will get the subtype of font, Type1, Type3, ...
@@ -599,10 +656,12 @@ public abstract class PDFont implements COSObjectable
      */
     public String getSubType()
     {
-        if (subtype == null) {
+        if (subtype == null)
+        {
             subtype = font.getNameAsString( COSName.SUBTYPE );
             type1Font = "Type1".equals(subtype);
             trueTypeFont = "TrueType".equals(subtype);
+            type0Font = "Type0".equals(subtype);
             typeFont = type1Font || "Type0".equals(subtype) || trueTypeFont;
         }
         return subtype;
@@ -612,17 +671,30 @@ public abstract class PDFont implements COSObjectable
      * Determines if the font is a type 1 font.
      * @return returns true if the font is a type 1 font
      */
-    protected boolean isType1Font() {
+    protected boolean isType1Font()
+    {
         getSubType();
         return type1Font;
     }
 
-    private boolean isTrueTypeFont() {
+    /**
+     * Determines if the font is a type 0 font.
+     * @return returns true if the font is a type 0 font
+     */
+    protected boolean isType0Font()
+    {
+        getSubType();
+        return type0Font;
+    }
+
+    private boolean isTrueTypeFont()
+    {
         getSubType();
         return trueTypeFont;
     }
 
-    private boolean isTypeFont() {
+    private boolean isTypeFont()
+    {
         getSubType();
         return typeFont;
     }
@@ -708,12 +780,12 @@ public abstract class PDFont implements COSObjectable
     /**
      * Set the widths of the characters code.
      *
-     * @param widths The widths of the character codes.
+     * @param widthsList The widths of the character codes.
      */
-    public void setWidths( List<Float> widths )
+    public void setWidths( List<Float> widthsList )
     {
-        this.widths = widths;
-        font.setItem( COSName.WIDTHS, COSArrayList.converterToCOSArray( this.widths ) );
+        widths = widthsList;
+        font.setItem( COSName.WIDTHS, COSArrayList.converterToCOSArray( widths ) );
     }
 
     /**
@@ -773,7 +845,7 @@ public abstract class PDFont implements COSObjectable
 
     /**
      * Determines the width of the given character.
-     * @param charCode the code of the given character 
+     * @param charCode the code of the given character
      * @return the width of the character
      */
     public float getFontWidth( int charCode )
@@ -783,14 +855,14 @@ public abstract class PDFont implements COSObjectable
         int lastChar = getLastChar();
         if (charCode >= firstChar && charCode <= lastChar)
         {
-            List<Float> widths = getWidths();
             // maybe the font doesn't provide any widths
+            getWidths();
             if (widths != null)
             {
                 width = widths.get(charCode-firstChar).floatValue();
             }
         }
-        else 
+        else
         {
             PDFontDescriptor fd = getFontDescriptor();
             if (fd instanceof PDFontDescriptorDictionary)
@@ -805,24 +877,24 @@ public abstract class PDFont implements COSObjectable
      * Determines if a font as a ToUnicode entry.
      * @return true if the font has a ToUnicode entry
      */
-    protected boolean hasToUnicode() 
+    protected boolean hasToUnicode()
     {
         return hasToUnicode;
     }
-    
+
     /**
      * Sets hasToUnicode to the given value.
-     * @param hasToUnicode the given value for hasToUnicode
+     * @param hasToUnicodeValue the given value for hasToUnicode
      */
-    protected void setHasToUnicode(boolean hasToUnicode)
+    protected void setHasToUnicode(boolean hasToUnicodeValue)
     {
-        this.hasToUnicode = hasToUnicode;
+        hasToUnicode = hasToUnicodeValue;
     }
     
-    public COSString createString(String text) throws IOException
-    {
-        return new COSString(text);
-        
-    }
-
+    /**
+     * Determines the width of the space character.
+     * @return the width of the space character
+     */
+    public abstract float getSpaceWidth();
+    
 }
